@@ -1033,7 +1033,7 @@ sub ParseH264Video($$)
             printf $out "  NAL Unit Type: 0x%x (%d bytes)\n",$nal_unit_type, length $buff;
             my %parms = ( Out => $out );
             $parms{MaxLen} = 96 if $verbose < 4;
-            Image::ExifTool::HexDump(\$buff, undef, %parms) if $verbose > 2;
+            HexDump(\$buff, undef, %parms) if $verbose > 2;
         }
         pos($$dataPt) = $pos = $nextPos;
 
