@@ -19,7 +19,7 @@ package Image::ExifTool::Shortcuts;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '1.47';
+$VERSION = '1.53';
 
 # this is a special table used to define command-line shortcuts
 # (documentation Notes may be added for these via %shortcutNotes in BuildTagLookup.pm)
@@ -140,8 +140,10 @@ $VERSION = '1.47';
         'MakerNoteKodak7',
         'MakerNoteKodak8a',
         'MakerNoteKodak8b',
+        'MakerNoteKodak8c',
         'MakerNoteKodak9',
         'MakerNoteKodak10',
+        'MakerNoteKodak11',
         'MakerNoteKodakUnknown',
         'MakerNoteKyocera',
         'MakerNoteMinolta',
@@ -150,6 +152,7 @@ $VERSION = '1.47';
         'MakerNoteNikon',
         'MakerNoteNikon2',
         'MakerNoteNikon3',
+        'MakerNoteNintendo',
         'MakerNoteOlympus',
         'MakerNoteOlympus2',
         'MakerNoteLeica',
@@ -169,6 +172,7 @@ $VERSION = '1.47';
         'MakerNotePhaseOne',
         'MakerNoteReconyx',
         'MakerNoteRicoh',
+        'MakerNoteRicoh2',
         'MakerNoteRicohText',
         'MakerNoteSamsung1a',
         'MakerNoteSamsung1b',
@@ -201,6 +205,14 @@ $VERSION = '1.47';
         'InteropIFD:RelatedImageWidth',
         'InteropIFD:RelatedImageHeight',
     ],
+    # standard tags used to define the color space of an image
+    # (useful to preserve color space when deleting all meta information)
+    ColorSpaceTags => [
+        'ExifIFD:ColorSpace',
+        'ExifIFD:Gamma',
+        'InteropIFD:InteropIndex',
+        'ICC_Profile',
+    ],
     # common metadata tags found in IFD0 of TIFF images
     CommonIFD0 => [
         # standard EXIF
@@ -223,6 +235,25 @@ $VERSION = '1.47';
         'IFD0:XPKeywords',
         'IFD0:XPSubject',
     ],
+    # large binary data tags which won't be loaded if excluded when extracting
+    LargeTags => [
+        'CanonVRD',
+        'DLOData',
+        'EXIF',
+        'ICC_Profile',
+        'IDCPreviewImage',
+        'ImageData',
+        'IPTC',
+        'JpgFromRaw',
+        'OriginalRawImage',
+        'OtherImage',
+        'PreviewImage',
+        'ThumbnailImage',
+        'TIFFPreview',
+        'XML',
+        'XMP',
+        'ZoomedPreviewImage',
+    ],
 );
 
 #------------------------------------------------------------------------------
@@ -284,6 +315,10 @@ In this example, MyShortcut is a shortcut for the CreateDate,
 EXIF:ExposureTime and Aperture tags, and MyAlias is a shortcut for
 FocalLengthIn35mmFormat.
 
+The target tag names may contain an optional group name prefix.  A group
+name applied to the shortcut will be ignored for any target tag with a group
+name prefix.
+
 =head1 AUTHOR
 
 Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
